@@ -1,7 +1,7 @@
 
-/*  $Header: /cvs/gnome/gcalctool/gcalctool/display.c,v 1.35 2003/12/05 20:53:25 richb Exp $
+/*  $Header: /cvs/gnome/gcalctool/gcalctool/display.c,v 1.46.2.1 2005/01/12 19:07:42 richb Exp $
  *
- *  Copyright (c) 1987-2003 Sun Microsystems, Inc. All Rights Reserved.
+ *  Copyright (c) 1987-2004 Sun Microsystems, Inc. All Rights Reserved.
  *           
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -29,79 +29,62 @@ static char *make_fixed(int *, int, int);
 
 
 /* Add in the thousand separators characters if required and if we are
- * currently in the decimal numeric base.
+ * currently in the decimal numeric base, use the "right" radix character.
  */
 
 void
-add_tsep()
+localize_number(char *dest, const char *src)
 {
-    if (v->show_tsep && v->base == DEC) {
-	char *dstp, *radixp, *srcp;
-	int n;
+    char tnum[MAX_LOCALIZED], *dstp;
+
+    if (!v->error && v->show_tsep && v->base == DEC) {
+	const char *radixp, *srcp;
+	int n, i;
+	size_t tsep_len;
 
 	/* Process the fractional part (if any). */
-	srcp = v->fnum + strlen(v->fnum) - 1;
-	dstp = v->tnum;
-	if ((radixp = strchr(v->fnum, v->radix_char)) != NULL) {
-            while (*srcp != v->radix_char) {
+	srcp = src + strlen(src) - 1;
+	dstp = tnum;
+	if ((radixp = strchr(src, '.')) != NULL) {
+            while (srcp != radixp) {
     		*dstp++ = *srcp--;
             }
-            *dstp++ = *srcp--;	   /* Copy over the radix char. */
+	    *dstp++ = *srcp--;
     	}
 
     	/* Process the integer part, add in thousand separators. */
+	tsep_len = strlen(v->tsep);
     	n = 0;
-    	while (srcp >= v->fnum) {
+    	while (srcp >= src) {
             *dstp++ = *srcp--;
             n++;
-            if (n == 3 && srcp >= v->fnum) {
-		*dstp++ = v->tsep_char;
+            if (n == 3 && srcp >= src && *srcp != '-') {
+	        for (i = tsep_len - 1; i >= 0; i--)
+		    *dstp++ = v->tsep[i];
 		n = 0;
             }
 	}
 	*dstp++ = '\0';
 
 	/* Move from scratch pad to fnum, reversing the character order. */
-	srcp = v->tnum + strlen(v->tnum) - 1;
-	dstp = v->fnum;
-	while (srcp >= v->tnum) {
+	srcp = tnum + strlen(tnum) - 1;
+	dstp = dest;
+	while (srcp >= tnum) {
             *dstp++ = *srcp--;
 	}
 	*dstp++ = '\0';
+    } else {
+	STRCPY(dest, src);
     }
-}
+    dstp = strchr(dest, '.');
+    if (dstp != NULL) {
+	size_t radix_len;
 
-
-/* Remove the thousands separators (if any) in-situ. */
-
-void
-remove_tsep(char *str) {
-    gboolean found = FALSE;
-    char *srcp = str;
-    char *dstp = str;
-
-/* If there are no thousands separators in the numeric string, just return
- * without doing anything. This is needed because the initial constant
- * definitions are potentially in read-only memory, and shouldn't be adjusted.
- */
-
-    while (*srcp != '\0') {
-        if (*srcp == v->tsep_char) {
-            found = TRUE;
-        }
-        srcp++;
-    }
-
-    if (found == TRUE) {
-        srcp = str;
-        while (*srcp != '\0') {
-            if (*srcp == v->tsep_char) {
-                srcp++;
-                continue;
-            }
-            *dstp++ = *srcp++;
-        }
-        *dstp++ = '\0';
+	radix_len = strlen(v->radix);
+	if (radix_len != 1) {
+	    memmove(dstp + radix_len, dstp + 1, strlen (dstp + 1) + 1);
+	}
+	MEMCPY(dstp, v->radix, radix_len);
     }
 }
 
@@ -131,12 +114,13 @@ clear_display(int initialise)
     i = 0;
     mpcim(&i, v->MPdisp_val);
     STRCPY(v->display, make_number(v->MPdisp_val, v->base, FALSE, FALSE));
-    set_display(v->display);
+    set_display(v->display, FALSE);
 
     if (initialise == TRUE) {
         v->show_paren = 0;
         v->opsptr     = 0;            /* Clear parentheses stacks. */
         v->numsptr    = 0;
+        v->noparens   = 0;
         set_hyp_item(FALSE);          /* Also clears v->hyperbolic. */
         set_inv_item(FALSE);          /* Also clears v->inverse. */
     }
@@ -173,6 +157,7 @@ initialise()
     v->error         = 0;           /* Currently no display error. */
     v->cur_op        = '?';         /* No arithmetic operator defined yet. */
     v->old_cal_value = '?';
+    v->pending       = 0;
     i = 0;
     mpcim(&i, v->MPresult);         /* No previous result yet. */
     mpcim(&i, v->MPlast_input);
@@ -180,7 +165,7 @@ initialise()
 
 
 /* Convert MP number to fixed number string in the given base to the
- * maximum number of characters specified.
+ * maximum number of digits specified.
  */
 
 static char *
@@ -203,7 +188,7 @@ make_fixed(int *MPnumber, int base, int cmax)
     mpcim(&basevals[base], MP1base);
 
     mppwr(MP1base, &v->accuracy, MP1);
-    SPRINTF(half, "0%c5", v->radix_char);
+    SPRINTF(half, "0.5"); /* FIXME: string constant if MPstr_to_num can get it */
     MPstr_to_num(half, DEC, MP2);
     mpdiv(MP2, MP1, MP1);
     mpadd(MPval, MP1, MPval);
@@ -224,7 +209,7 @@ make_fixed(int *MPnumber, int base, int cmax)
 
     while (ndig-- > 0) {
         if (ddig-- == 0) {
-            *optr++ = v->radix_char;
+	    *optr++ = '.';
         }
         mpmul(MPval, MP1base, MPval);
         mpcmi(MPval, &dval);
@@ -246,14 +231,12 @@ make_fixed(int *MPnumber, int base, int cmax)
         while (*optr == '0') {
             optr--;
         }
-        if (*optr != v->radix_char) {
-            optr++;
-        }
+	if (optr < v->fnum || *optr != '.') {
+	    optr++;
+	}
         *optr = '\0';
     }
 
-    add_tsep();
-
     return(v->fnum);
 }
 
@@ -293,7 +276,7 @@ make_number(int *MPnumber, int base, BOOLEAN mkFix, BOOLEAN ignoreError)
 static char *
 make_eng_sci(int *MPnumber, int base)
 {
-    char half[4], fixed[MAX_DIGITS+1], *optr;
+    char half[4], fixed[MAX_DIGITS], *optr;
     int MP1[MP_SIZE], MPatmp[MP_SIZE], MPval[MP_SIZE];
     int MP1base[MP_SIZE], MP3base[MP_SIZE], MP10base[MP_SIZE];
     int i, dval, len, n;
@@ -367,7 +350,7 @@ make_eng_sci(int *MPnumber, int base)
         *optr++ = '+';
     }
  
-    SPRINTF(half, "0%c5", v->radix_char);
+    SPRINTF(half, "0.5");
     MPstr_to_num(half, DEC, MP1);
     mpaddi(MP1, &exp, MPval);
     n = 1;
@@ -395,7 +378,8 @@ make_eng_sci(int *MPnumber, int base)
 }
 
 
-/* Convert string into an MP number. */
+/* Convert string into an MP number, in the given base
+ */
 
 void
 MPstr_to_num(char *str, enum base_type base, int *MPval)
@@ -410,8 +394,6 @@ MPstr_to_num(char *str, enum base_type base, int *MPval)
     mpcim(&i, MPval);
     mpcim(&basevals[(int) base], MPbase);
 
-    remove_tsep(str);
-
     optr = str;
     while ((inum = char_val(*optr)) >= 0) {
         mpmul(MPval, MPbase, MPval);
@@ -419,12 +401,14 @@ MPstr_to_num(char *str, enum base_type base, int *MPval)
         optr++;
     }
 
-    if (*optr == v->radix_char) {
-        for (i = 1; (inum = char_val(*++optr)) >= 0; i++) {
+    if (*optr == '.') {
+        optr++;
+        for (i = 1; (inum = char_val(*optr)) >= 0; i++) {
             mppwr(MPbase, &i, MP1);
             mpcim(&inum, MP2);
             mpdiv(MP2, MP1, MP1);
             mpadd(MPval, MP1, MPval);
+	    optr++;
         }
     }
 
@@ -468,7 +452,7 @@ paren_disp(char c)
  */
 
     n = strlen(v->display);
-    if (c == -1) {         /* Is it a Delete character? */
+    if (c == -1 || c == 127) {         /* Is it a Delete character? */
         v->noparens = v->pending = v->opsptr = v->numsptr = 0;
         v->cur_op = '?';
         i = 0;
@@ -479,7 +463,10 @@ paren_disp(char c)
         if (!n) {
             return;
         }
-        if (v->display[n-1] == '(') {
+
+        if (v->display[n-1] == ')') {
+            v->noparens++;
+        } else if (v->display[n-1] == '(') {
             v->noparens--;
             if (!v->noparens) {
                 v->pending = v->opsptr = v->numsptr = 0;
@@ -516,7 +503,7 @@ paren_disp(char c)
 
     n = (n < MAX_DIGITS) ? 0 : n - MAX_DIGITS;
     v->show_paren = 1;       /* Hack to get set_display to really display it. */
-    set_display(&v->display[n]);
+    set_display(&v->display[n], FALSE);
     v->show_paren = 0;
 }
 
@@ -575,6 +562,6 @@ show_display(int *MPval)
 {
     if (!v->error) {
         STRCPY(v->display, make_number(MPval, v->base, TRUE, FALSE));
-        set_display(v->display);
+        set_display(v->display, FALSE);
     }
 }
